//                               -*- Mode: c++ -*- 
// SourceUnit.hh --- Source Unit sub-hierarchy
// Author          : Timothy Lethbridge
// Created On      : Fri Nov 22 13:03:38 1996
//


# ifndef SOURCEUNIT_H
# define SOURCEUNIT_H

# include "newDB.hh"
# include "Definition.hh"
# include "ReferenceExistence.hh"


class SourceUnit: public SoftwareObject {
// --------------------------------------------------
protected:
  List<SourceWithinFile *>     _contains;
  // converse: SourceWithinFile -> containingSource
  List<DataUseExistence *>     _refersToData;
  // converse: DataUseExistence -> foundInSource
  List<CommentTermExistence *> _refersToCommentTerm;
  // converse: DataUseExistence -> foundInSource
  List<StandaloneDefinition *> _defines;
  // converse: StandaloneDefinition -> containingSource
  List<RoutineCallExistence *> _calls;
  // converse: RoutineCallExistence -> (List) potentiallyCalledBy

public:
  SourceUnit() { the_DB->incCount( classType()); };

  List<SourceWithinFile *>     *contains() { return(&_contains); };
  List<DataUseExistence *>     *refersToData() { return(&_refersToData); };
  List<CommentTermExistence *> *refersToCommentTerm()
      { return(&_refersToCommentTerm); };
  List<StandaloneDefinition *> *defines() { return(&_defines); };
  List<RoutineCallExistence *> *calls() { return(&_calls); };

  void addContainedSource( SourceWithinFile *containedSource) {
	addInSet( contains(), containedSource); }
  void addReference( ReferenceExistence *usedEntity, bool isComment);
  void addDefinition( StandaloneDefinition *defined) {
	addInSet( defines(), defined); };
  void addCalledRoutine(RoutineCallExistence *rtnCalled) {
	addInSet( calls(), rtnCalled); };

  virtual VoidIterator<SoftwareObject*> *defines(class_t type);
  virtual VoidIterator<SoftwareObject*> *refersTo(class_t type);
  class_t classType() { return(_sourceunit); };
  bool is_a(class_t aType) {
	return( (aType == _sourceunit) || SoftwareObject::is_a(aType)); };

  virtual void sortLists() {
	sortTheList((List<SoftwareObject*>*)&_contains);
	sortTheList((List<SoftwareObject*>*)&_refersToData);
	//	sortTheList((List<SoftwareObject*>*)&_refersToCommentTerm);
	sortTheList((List<SoftwareObject*>*)&_defines);
	SoftwareObject::sortLists(); };

# ifdef DEBUG
  void hello();
# endif
};


class SourceFile: public SourceUnit {
// --------------------------------------------------
protected:
  Name *_fileName;
  // converse: Name -> (List) namedFiles
  char *_path;
  int   _version;
  int   _dateChanged;
  int   _size;
  List<FileInclusionExistence *> _includes;
  // converse: FileInclusionExistence -> (List) potentiallyIncludedIn

public:
  SourceFile(char *fileName, int vers, char *path, int dChange, int size);

  Name *baseName()   const { return( _fileName); };
  char *pathName()   const { return( _path); };
  char *fullName()   const { /* to be done */ };
  char *objectName() const { return( baseName()->string()); };
  int version()      const { return( _version); };
  int dateChanged()  const { return( _dateChanged); }
  int size()         const { return( _size); };
  List<FileInclusionExistence *> *includes() { return(&_includes); };

  VoidIterator<SoftwareObject*> *refersTo(class_t type);
  void addIncludedFile( FileInclusionExistence *includedFile) {
	addInSet( includes(), includedFile); };
  class_t classType() { return(_sourcefile); };
  bool is_a(class_t aType) {
	return( (aType ==_sourcefile ) || SourceUnit::is_a(aType)); };

  virtual void sortLists() {
	sortTheList((List<SoftwareObject*>*)&_includes);
	SourceUnit::sortLists(); };

# ifdef DEBUG
  void hello();
# endif
};


class SourceWithinFile: public SourceUnit {
// --------------------------------------------------
protected:
  SourceUnit *_containingSource;
  // converse: SourceUnit -> (List) contains
  int         _startChar, _endChar;

public:
  SourceWithinFile(SourceUnit *containerToBe, int startPos, int endPos);

  SourceUnit *containingSource() { return( _containingSource); };
  int startChar() { return( _startChar); };
  int endChar() { return( _endChar); };

  class_t classType() { return(_sourcewithinfile); };
  bool is_a(class_t aType) {
	return( (aType == _sourcewithinfile) || SourceUnit::is_a(aType)); };

# ifdef DEBUG
  void hello();
# endif
};


class RoutineSource: public SourceWithinFile {
// --------------------------------------------------
protected:
  Name *_routineName;
  // converse: Name -> (List) namedRoutines
  bool  _isEntry;
  TypeUseExistence *_returnType;
  // converse: TypeUseExistence -> (List) usedAsReturnTypeOf
  // List<DatumDef *>             _formalArgs;
  // converse: StandaloneDefinition -> containingSource

public:
  RoutineSource( SourceUnit *containerToBe,
				 int   startPos,
				 int   endPos,
				 char *rtnName,
				 bool  entry_p,
				 TypeUseExistence *returns);

  Name *routineName() const { return( _routineName); };
  char *objectName()  const { return( routineName()->string()); };
  bool isEntry()      const { return( _isEntry); };
  TypeUseExistence *returnType() const {return( _returnType); };
  bool isFunction()   const { return( returnType() != NULL); };
  bool isProcedure()  const { return( returnType() == NULL); };

  VoidIterator<SoftwareObject*> *definedBy(class_t type);
  VoidIterator<SoftwareObject*> *refersTo(class_t type);
  List<DatumDef *> *formalArgs();
  //  void addFormalArg(DatumDef *formalArg);
  class_t classType() { return(_routinesource); };
  bool is_a(class_t aType) {
	return( (aType == _routinesource) || SourceWithinFile::is_a(aType)); };

  virtual void sortLists() {
	sortTheList((List<SoftwareObject*>*)&_calls);
	SourceWithinFile::sortLists(); };

# ifdef DEBUG
  void hello();
# endif
};


class SourceBlock: public SourceWithinFile {
// --------------------------------------------------
public:
  SourceBlock( SourceUnit *containerToBe, int startPos, int endPos)
	: SourceWithinFile(containerToBe,startPos,endPos)
 {the_DB->incCount(classType());};

  class_t classType() { return(_sourceblock); };
  bool is_a(class_t aType) {
	return( (aType == _sourceblock) || SourceWithinFile::is_a(aType)); };
};

#endif
