/*                              -*- Mode: C++ -*-
 * write-API.hh --- API to create the new DB
 * Author          : Nicolas Anquetil
 * Created On      : Thu Dec 12 17:22:14 1996
 */

/* This file may be readable in C, beware of the comments */

/* Timothy Lethbridge --------------------------------------------------

   This file defines the functions to be used for creating a cdb database.
   Normally these functions would be called by a parser, however they can
   also be called by a program that converts one database format into a
   cdb database.

   A cdb database contains detailed information about a body of source
   code contained in files. The database is designed to be used by
   a program that helps software engineers understand that code.
   Currently the database can contain most of the important information
   that one might encounter in a procedural program.

   Note that this file is designed to be called from a C program. Our
   database is written in C++, so we also make provisions for it to be
   called from that language. However, if external users want to use C++
   then you will also have to include several other header files listed
   below (and thus your work becomes more complex).

   This package is designed to run on Solaris machines, although
   it may work on SunOs and other unix machines. The database package
   relies on the existence of the unix mmap utility.

   Conventionally, databases created by this packages have the suffix .mf
   Once the database is created, it can be read by a program that calls
   functions in the file read-API.hh. Note that cdb databases are
   designed to be write-once/read-many. There is no provision for
   concurrency control for multiple writers. There is also no provision
   for altering a database, only adding to it. Eventually, we plan to
   provide 'local overlay' databases that will mask parts of a main
   database, but this facility is not ready yet.

   Any program that reads or writes a cdb database must link to a unix
   shared object (CDB2.so) that contains the accessing functions. If the
   shared object changes (due to an update in the database format), then
   the database has to be rebuilt.

   Currently there is a limit on the size of the database of about 1.5GB.
   Let us know if this is a problem. If you need to process more than
   1.5 GB, the best way to do it is to create separate databases for
   different subsystems.

   Note that this file is part of a research project. Names, arguments,
   or anything else may change without notice. However, we will endeavour
   to make changes as infrequently as possible.
*/

# ifndef WRITEAPI_H
# define WRITEAPI_H

# include "c++_c.h"

# ifdef __cplusplus
# include "utils.hh"
# include "SourceUnit.hh"
# include "Definition.hh"
# include "ReferenceExistence.hh"
# endif

/* synonyms for generic database functions (defined in `filemem.h') */
/* called to initiate and terminate creation of a database */
# define cdbInitGrowableFileMem initGrowableFileMem
# ifndef cdbTerminateFileMem
# define cdbTerminateFileMem    terminateFileMem
# endif

/* As explained in c++_c.h, for C, these functions will all return\
(void *) */

/* ----- Called whenever a new file is being processed */
EXTERNC SourceFile *cdbCreateSourceFile(
  char *name,                   /* without the path */
  int version,                  /* the version of the file */
  char *path,                   /* whereever the file is stored */
  int dateChanged,              /* days since Jan 1, 1900 ??? */ 
  int size                      /* in bytes (0 means don't know) */
);

/* ----- Called whenever a new routine is encountered in the source */
EXTERNC RoutineSource *cdbCreateRoutineSource(
  SourceUnit *containingSource, /* normally a file, but could be a routine */
  char *name,
  char *optReturnTypeName,
  int startChar,                /* offset within the file */
  int endChar,                  /* offset within the file */
  bool isEntry                  /* is this globally known or not */
);

/* ----- Called whenever a definition of a data item is encountered */
EXTERNC DatumDef *cdbCreateDatumDef(
  SourceUnit *fileOrRoutine,    /* innermost scope */
  char *name,                   /* of whatever is being defined */
  bool isFormalArgument,        /* argument to a routine? */
  bool isConstant,
  char *optTypeName,            /* if it has a type */
  int startChar,                /* offset within file */
  int endChar                   /* offset within file */
);

/* ----- Called whenever a type is defined */ 
EXTERNC TypeDef *cdbCreateTypeDef(
  SourceUnit *containingSource, /* innermost scope in which defined */
  char *name,                   /* of the type */
  char *optTypeName,            /* type of this type */
  bool isRecord,                /* record type, c-struct etc.? */
  bool isEnumerated,            /* enumerated type? */
  int startChar,                /* offset within file */
  int endChar                   /* offset within file */
);

/* ----- Called whenever a field definition of a record is encountered */
EXTERNC Field *cdbCreateField(
  RecordTypeDef *record,        /* result of cdbCreateTypeDef */
  char *name,                   /* of the field */
  char *optTypeName             /* datatype of the field */
);

/* ----- Called whenever an enumeration constant is defined within an
   enumeration type */
EXTERNC EnumerationConst *cdbCreateEnumerationConst(
  EnumeratedTypeDef *enumeration, /* result of cdbCreateTypeDef */
  char *name                      /* of the Enumeration constant */
);

/* ----- Called whenever a call to a routine is encountered */
EXTERNC RoutineCallExistence *cdbRegisterRoutineCall(
  SourceUnit *caller,
  char *calledRoutineName
);

/* ----- Called whenever a file inclusion or importation is encountered */
EXTERNC FileInclusionExistence *cdbRegisterFileInclusion(
  SourceFile *includerFile,
  char *includedFileName);

/* called whenever a word in a comment is found. Ordinary English words
   such as 'and', 'it', 'the' should be skipped */
EXTERNC CommentTermExistence *cdbRegisterCommentTerm(
  SourceUnit *containingSource,   /* Innermost scope */
  char *name,                     /* The word itself */
  CommentTermExistence *optPrevTerm, /* If this word immediately
      follows some other term, then pass that term */
  bool isConditionalCompData      /* True if this comment word is
      actually read by the compiler for some reason */
);

/* called whenever a reference is made to a data item, e.g. as an argument
   to a routine, on either side of an assignment etc. */
EXTERNC DataUseExistence *cdbRegisterDataUse(
  SourceUnit *containingSource,   /* Innermost scope */
  char *dataName,                 /* Data item being used */
  DataUseExistence *optPreviousComponent, /* If this data item is a
    second or subsequent part of a multi-part name (e.g. xxx.yyy)
    then pass the previous component */
  bool isManifestConstant,        /* Happens to be a manifest constant such
      as a number, character string etc */
  bool isTypeUse                  /* The data item happens to be a type */
);

# endif
